<?php
if (!defined('_PS_VERSION_'))
  exit;

require_once __DIR__ . "/vendor/php-curl/src/cURL.php";
require_once __DIR__ . "/vendor/php-curl/src/cURLException.php";
require_once __DIR__ . "/vendor/php-curl/src/Request.php";
require_once __DIR__ . "/vendor/php-curl/src/Response.php";

class Transactionale extends Module
{
    public function __construct()
    {
        $this->name = 'transactionale';
        $this->tab = 'emailing';
        $this->version = '1.7.3';
        $this->author = 'Transactionale';
        $this->need_instance = 0;
        $this->ps_version_compliancy = array('min' => '1.5', 'max' => '1.6');
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Transactionale');
        $this->description = $this->l('Transactionale integration for Advertisers and Publishers');
    }

    public function install()
    {
        if (Shop::isFeatureActive())
            Shop::setContext(Shop::CONTEXT_ALL);

        if(!parent::install() ||
            !$this->registerHook('displayTop') ||
            !$this->registerHook('displayOrderConfirmation') ||
            !Configuration::updateValue('TR_API_KEY', '') ||
            !Configuration::updateValue('TR_COUNTRY', '') ||
            !Configuration::updateValue('TR_IGNORE_OPTIN', '0') ||
            !Configuration::updateValue('TR_HOST', 'https://www.transactionale.com') ||
            !Configuration::updateValue('TR_TOUCHPOINT_ORDER_ENABLED', 0) ||
            !Configuration::updateValue('TR_TOUCHPOINT_ORDER_TEXT', '')
        )
            return false;


        return true;
    }

    public function uninstall()
    {
        if (Shop::isFeatureActive())
            Shop::setContext(Shop::CONTEXT_ALL);

        if (!parent::uninstall() ||
            !Configuration::deleteByName('TR_API_KEY') ||
            !Configuration::deleteByName('TR_COUNTRY') ||
            !Configuration::deleteByName('TR_IGNORE_OPTIN') ||
            !Configuration::deleteByName('TR_HOST') ||
            !Configuration::deleteByName('TR_TOUCHPOINT_ORDER_ENABLED') ||
            !Configuration::deleteByName('TR_TOUCHPOINT_ORDER_TEXT')
        )
        return false;

      return true;
    }

    public function transactionaleIsEnabled()
    {
        return true;
    }

    public function isEnabledAdvertiser()
    {
        return $this->transactionaleIsEnabled();
    }

    public function isEnabledDistributor()
    {
        return $this->transactionaleIsEnabled();
    }

    public function getContent()
    {
        if(Shop::getContext() !== Shop::CONTEXT_SHOP)
        {
            return $this->displayError($this->l('Please select a single shop!'));
        }

        $output = null;

        if (Tools::isSubmit('submit'.$this->name))
        {
            $api_key = strval(Tools::getValue('TR_API_KEY'));
            $country = strval(Tools::getValue('TR_COUNTRY'));
            $ignore_opt_in = Tools::getValue('TR_IGNORE_OPTIN');
            $enabled = 1;
            $touchpoint_id = intval(Tools::getValue('TR_TOUCHPOINT_ORDER_ENABLED'));

            if (_PS_MODE_DEV_ === true) {
                $tr_host = strval(Tools::getValue('TR_HOST'));
            }
            
            if (!$api_key
              || empty($api_key)
              || !Validate::isGenericName($api_key)
              || (_PS_MODE_DEV_ === true && empty($tr_host)))
                $output .= $this->displayError($this->l('Invalid Configuration value'));
            else
            {
                Configuration::updateValue('TR_API_KEY', $api_key);
                Configuration::updateValue('TR_COUNTRY', $country);
                Configuration::updateValue('TR_IGNORE_OPTIN', $ignore_opt_in);
                Configuration::updateValue('TR_TOUCHPOINT_ORDER_ENABLED', $touchpoint_id);
                Configuration::updateValue('TR_TOUCHPOINT_ORDER_TEXT', strval(Tools::getValue('TR_TOUCHPOINT_ORDER_TEXT')));

                if (_PS_MODE_DEV_ === true) {
                    Configuration::updateValue('TR_HOST', $tr_host);
                }
                
                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }

        return $output.$this->displayForm().$this->displayTransactionaleSetup();
    }

    public function displayForm()
    {
        // Get default language
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
        $TR_TOUCHPOINT_ORDER_ENABLED_options = $this->getTouchpointOptions();

        // Init Fields form array
        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Settings'),
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('API Key'),
                    'name' => 'TR_API_KEY',
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Country Code'),
                    'name' => 'TR_COUNTRY',
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Ignore opt-in'),
                    'name' => 'TR_IGNORE_OPTIN',
                    'required' => false,
                    'options' => array(
                        'query' => array(
                            array(
                                'name' => $this->l('Yes'),
                                'id_option' => '1'
                            ),
                            array(
                                'name' => $this->l('No'),
                                'id_option' => '0'
                            ),
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',                              // This is a <select> tag.
                    'label' => $this->l('Enable order confirmation touchpoint'),         // The <label> for this <select> tag.
                    'name' => 'TR_TOUCHPOINT_ORDER_ENABLED',                     // The content of the 'id' attribute of the <select> tag.
                    'required' => true,                              // If set to true, this option must be set.
                    'options' => array(
                        'query' => $TR_TOUCHPOINT_ORDER_ENABLED_options,                           // $options contains the data itself.
                        'id' => 'id_option',                           // The value of the 'id' key must be the same as the key for 'value' attribute of the <option> tag in each $options sub-array.
                        'name' => 'name'                               // The value of the 'name' key must be the same as the key for the text content of the <option> tag in each $options sub-array.
                    )
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Order confirmation touchpoint text'),
                    'name' => 'TR_TOUCHPOINT_ORDER_TEXT',
                    'size' => 20,
                    'required' => false
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'btn'
            )
        );

        if (_PS_MODE_DEV_ === true) {
            $fields_form[0]['form']['input'][] = array(
                    'type' => 'text',                              // This is a <select> tag.
                    'label' => $this->l('Transactionale Host URL'),         // The <label> for this <select> tag.
                    'name' => 'TR_HOST',                     // The content of the 'id' attribute of the <select> tag.
                    'required' => true,                              // If set to true, this option must be set.
                    'size' => 200,
                );
        }

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        // Language
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = array(
            'save' =>
            array(
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                '&token='.Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        // Load current value
        $helper->fields_value['TR_API_KEY'] = Configuration::get('TR_API_KEY');
        $helper->fields_value['TR_COUNTRY'] = Configuration::get('TR_COUNTRY');
        $helper->fields_value['TR_HOST'] = Configuration::get('TR_HOST');
        $helper->fields_value['TR_IGNORE_OPTIN'] = Configuration::get('TR_IGNORE_OPTIN');
        $helper->fields_value['TR_TOUCHPOINT_ORDER_ENABLED'] = Configuration::get('TR_TOUCHPOINT_ORDER_ENABLED');
        $helper->fields_value['TR_TOUCHPOINT_ORDER_TEXT'] = Configuration::get('TR_TOUCHPOINT_ORDER_TEXT');

        return $helper->generateForm($fields_form);
    }

    public function displayTransactionaleSetup()
    {
        $this->smarty->assign('form_action', AdminController::$currentIndex.'&configure='.$this->name.'&verify'.$this->name.
            '&token='.Tools::getAdminTokenLite('AdminModules'));
        $this->smarty->assign('api_key', Configuration::get('TR_API_KEY'));
        $this->smarty->assign('tr_host', Configuration::get('TR_HOST'));

        return $this->display(__FILE__, '/views/templates/admin/setup.tpl');
    }

    private function callAPI($method, $endpoint, $payload=[])
    {
        $api_key = Configuration::get('TR_API_KEY');
        $tr_host = Configuration::get('TR_HOST');

        $completeEndpointUrl = $tr_host . "/api/v1/" . $endpoint;

        $headers = array(
            "X-Authorization" => $api_key,
            "Content-Type" => "application/json",
            "Accept" => "application/json",
            "X-Platform" => 'Prestashop',
            "X-Platform-Version" =>  _PS_VERSION_,
            "X-Integration-Version" => $this->version
        );

        $curl = new anlutro\cURL\cURL;
        $curl->setDefaultHeaders($headers);

        try {
            switch($method) {
                case 'get':
                    $response = $curl->get($completeEndpointUrl);
                    break;
                case 'post':
                    $response = $curl->rawPost($completeEndpointUrl, json_encode($payload));
                    break;
                default:
                    return null;
            }
            
            if($response->statusCode <= 300) {
                if (_PS_MODE_DEV_ === true) {
                    Logger::addLog('Transactionale success: ' . $response->body, 1, $response->statusCode);
                }
                return $response;
            }

            Logger::addLog('Transactionale error: ' . $response->body, 2, $response->statusCode);

            return $response;
        } catch(Exception $err) {
            Logger::addLog('Transactionale exception: ' . $err->getMessage(), 3);
            return $err->getMessage();
        }
    }

    public function getTouchpoints()
    {
        $api_key = Configuration::get('TR_API_KEY');
        $tr_host = Configuration::get('TR_HOST');

        if(!$api_key) {
            return array();
        }

        $response = $this->callAPI('get', 'plugins/touchpoints');

        if($response->body) {
            return json_decode($response->body);
        }

        Logger::addLog('Transactionale error: Could not get touchpoints: ' . (string) $response->body, 3);
        return array();
    }

    public function getTouchpointSnippet($touchpoint_id)
    {
        $api_key = Configuration::get('TR_API_KEY');
        $tr_host = Configuration::get('TR_HOST');

        if(!$api_key) {
            return '';
        }

        $response = $this->callAPI('get', 'plugins/touchpoints/' . $touchpoint_id);

        if($response->success) {
            $data = json_decode($response->body);
            return $data->snippet;
        }

        Logger::addLog('Transactionale error: Could not get touchpoint detail: ' . (string) $response->body, 3);
        return '';
    }

    public function getTouchpointOptions()
    {
        $touchpoints = $this->getTouchpoints();
        $options = array(
            array(
                'id_option' => '',
                'name' => 'Disabled'
            )
        );

        foreach($touchpoints as $touchpoint) {
            $options[] = array(
                'id_option' => $touchpoint->id,
                'name' => "[{$touchpoint->country_code}] {$touchpoint->name}",
            );
        }

        return $options;
    }

    public function hookDisplayOrderConfirmation($params)
    {
        $order = $params['objOrder'];
        $customer = new Customer((int) $order->id_customer);
        $cart = new Cart((int) $order->id_cart);
        $shippingAddress = new Address((int)$cart->id_address_delivery);
        $shippingCountry = new Country($shippingAddress->id_country);

        $payload = array(
            'event' => 'trackTransaction',
            'transaction' => array(
                'id' => $order->reference,
                'total' => $params['total_to_pay'],
                'currency' => $params['currencyObj']->iso_code,
            ),
            'address' => array(
                'address' => $shippingAddress->address1,
                'address2' => $shippingAddress->address2,
                'city' => $shippingAddress->city,
                'postalCode' => $shippingAddress->postcode,
                'country' => $shippingCountry->iso_code,
            ),
            'customer' => array(
                'firstName' => $customer->firstname,
                'lastName' => $customer->lastname,
                'email' => $customer->email,
                'gender' => $this->getGenderCode($customer),
                'birthDate' => $customer->birthday,
            )
        );

        if (!boolval(Configuration::get('TR_IGNORE_OPTIN'))) {
            $payload['customer']['optin'] = boolval($customer->optin);
        }

        $webTouchpointId = Configuration::get('TR_TOUCHPOINT_ORDER_ENABLED');

        if ($webTouchpointId) {
            $payload['touchpointType'] = 'web';
            $payload['touchpointId'] = $webTouchpointId;
        }

        $this->smarty->assign('tr_payload', json_encode($payload));
        $this->smarty->assign('tr_integrationVersion', $this->version);
        $this->smarty->assign('tr_touchpointText', Configuration::get('TR_TOUCHPOINT_ORDER_TEXT'));
        $this->smarty->assign('tr_hasWebTouchpoint', $webTouchpointId ? true : false);
        return $this->display(__FILE__, '/views/templates/hooks/displayOrderConfirmation.tpl');
    }

    public function getGenderCode($customer)
    {
        switch($customer->id_gender) {
            case 1:
                $gender_code = 'm';
                break;
            case 2:
                $gender_code = 'f';
                break;
            default:
                $gender_code = null;
        }

        return $gender_code;
    }

    public function hookDisplayTop()
    {
        $this->smarty->assign('tr_debug', _PS_MODE_DEV_);
        $this->smarty->assign('tr_apiKey', Configuration::get('TR_API_KEY'));
        $this->smarty->assign('tr_country', Configuration::get('TR_COUNTRY'));
        $this->smarty->assign('tr_platform', 'Prestashop');
        $this->smarty->assign('tr_platformVersion', _PS_VERSION_);
        $this->smarty->assign('tr_integrationVersion', $this->version);
        return $this->display(__FILE__, '/views/templates/hooks/displayTop.tpl');
    }
}
